@@ -24,7 +24,6 @@
 #include "gtksourceview-i18n.h"
 #include "gtksourceview-marshal.h"
 #include "gtksourcegutterrenderer.h"
-#include "gtksourcegutterrenderer-private.h"
 
 #include <gobject/gvaluecollector.h>
 
@@ -49,7 +48,7 @@
  *
  */
 
-#define GTK_SOURCE_GUTTER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_GUTTER, GtkSourceGutterPrivate))
+#define GTK_SOURCE_GUTTER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_TYPE_SOURCE_GUTTER, GtkSourceGutterPrivate))
 
 /* Properties */
 enum
@@ -72,7 +71,6 @@ typedef struct
 	guint size_changed_handler;
 	guint notify_xpad_handler;
 	guint notify_ypad_handler;
-	guint notify_visible_handler;
 } Renderer;
 
 enum
@@ -84,7 +82,6 @@ enum
 	LEAVE_NOTIFY_EVENT,
 	QUERY_TOOLTIP_EVENT,
 	REALIZE,
-	STYLE_UPDATED,
 	LAST_EXTERNAL_SIGNAL
 };
 
@@ -94,6 +91,7 @@ struct _GtkSourceGutterPrivate
 	GtkTextWindowType window_type;
 	GtkOrientation orientation;
 
+	gint size;
 	GList *renderers;
 
 	gint xpad;
@@ -134,9 +132,6 @@ static gboolean on_view_query_tooltip (GtkSourceView   *view,
                                        GtkTooltip      *tooltip,
                                        GtkSourceGutter *gutter);
 
-static void on_view_style_updated (GtkSourceView    *view,
-                                   GtkSourceGutter  *gutter);
-
 static void do_redraw (GtkSourceGutter *gutter);
 static void update_gutter_size (GtkSourceGutter *gutter);
 
@@ -163,29 +158,25 @@ on_renderer_notify_padding (GtkSourceGutterRenderer *renderer,
 	update_gutter_size (gutter);
 }
 
-static void
-on_renderer_notify_visible (GtkSourceGutterRenderer *renderer,
-                            GParamSpec              *spec,
-                            GtkSourceGutter         *gutter)
-{
-	update_gutter_size (gutter);
-}
-
 static Renderer *
 renderer_new (GtkSourceGutter         *gutter,
-              GtkSourceGutterRenderer *renderer,
-              gint                     position)
+              GType                    gtype,
+              gint                     position,
+              guint                    num_parameters,
+              GParameter              *parameters)
 {
 	Renderer *ret = g_slice_new0 (Renderer);
 
-	ret->renderer = g_object_ref_sink (renderer);
+	ret->renderer = g_object_newv (gtype, num_parameters, parameters);
+
+	if (G_IS_INITIALLY_UNOWNED (ret->renderer))
+	{
+		g_object_ref_sink (ret->renderer);
+	}
+
 	ret->position = position;
 	ret->prelit = -1;
 
-	_gtk_source_gutter_renderer_set_view (renderer,
-	                                      GTK_TEXT_VIEW (gutter->priv->view),
-	                                      gutter->priv->window_type);
-
 	ret->size_changed_handler =
 		g_signal_connect (ret->renderer,
 		                  "notify::size",
@@ -210,12 +201,6 @@ renderer_new (GtkSourceGutter         *gutter,
 		                  G_CALLBACK (on_renderer_notify_padding),
 		                  gutter);
 
-	ret->notify_visible_handler =
-		g_signal_connect (ret->renderer,
-		                  "notify::visible",
-		                  G_CALLBACK (on_renderer_notify_visible),
-		                  gutter);
-
 	return ret;
 }
 
@@ -234,13 +219,6 @@ renderer_free (Renderer *renderer)
 	g_signal_handler_disconnect (renderer->renderer,
 	                             renderer->notify_ypad_handler);
 
-	g_signal_handler_disconnect (renderer->renderer,
-	                             renderer->notify_visible_handler);
-
-	_gtk_source_gutter_renderer_set_view (renderer->renderer,
-	                                      NULL,
-	                                      GTK_TEXT_WINDOW_PRIVATE);
-
 	g_object_unref (renderer->renderer);
 	g_slice_free (Renderer, renderer);
 }
@@ -257,7 +235,8 @@ gtk_source_gutter_dispose (GObject *object)
 	GtkSourceGutter *gutter = GTK_SOURCE_GUTTER (object);
 	gint i;
 
-	g_list_free_full (gutter->priv->renderers, (GDestroyNotify)renderer_free);
+	g_list_foreach (gutter->priv->renderers, (GFunc)renderer_free, NULL);
+	g_list_free (gutter->priv->renderers);
 
 	if (gutter->priv->view)
 	{
@@ -355,12 +334,6 @@ set_view (GtkSourceGutter *gutter,
 		                  "realize",
 		                  G_CALLBACK (on_view_realize),
 		                  gutter);
-
-	gutter->priv->signals[STYLE_UPDATED] =
-		g_signal_connect (view,
-		                  "style-updated",
-		                  G_CALLBACK (on_view_style_updated),
-		                  gutter);
 }
 
 static void
@@ -520,8 +493,6 @@ gtk_source_gutter_constructed (GObject *object)
 	{
 		gutter->priv->orientation = GTK_ORIENTATION_VERTICAL;
 	}
-
-	G_OBJECT_CLASS (gtk_source_gutter_parent_class)->constructed (object);
 }
 
 static void
@@ -546,7 +517,7 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 	                                 g_param_spec_object ("view",
 	                                                      _("View"),
 	                                                      _("The gutters' GtkSourceView"),
-	                                                      GTK_SOURCE_TYPE_VIEW,
+	                                                      GTK_TYPE_SOURCE_VIEW,
 	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	/**
@@ -630,7 +601,7 @@ GtkSourceGutter *
 gtk_source_gutter_new (GtkSourceView     *view,
                        GtkTextWindowType  type)
 {
-	return g_object_new (GTK_SOURCE_TYPE_GUTTER,
+	return g_object_new (GTK_TYPE_SOURCE_GUTTER,
 	                     "view", view,
 	                     "window_type", type,
 	                     NULL);
@@ -653,7 +624,7 @@ gtk_source_gutter_new (GtkSourceView     *view,
 GdkWindow *
 gtk_source_gutter_get_window (GtkSourceGutter *gutter)
 {
-	g_return_val_if_fail (GTK_SOURCE_IS_GUTTER (gutter), NULL);
+	g_return_val_if_fail (GTK_IS_SOURCE_GUTTER (gutter), NULL);
 	g_return_val_if_fail (gutter->priv->view != NULL, NULL);
 
 	return gtk_text_view_get_window (GTK_TEXT_VIEW (gutter->priv->view),
@@ -663,35 +634,195 @@ gtk_source_gutter_get_window (GtkSourceGutter *gutter)
 /**
  * gtk_source_gutter_insert:
  * @gutter: a #GtkSourceGutter.
- * @renderer: a gutter renderer (must inherit from #GtkSourceGutterRenderer).
- * @position: the renderer position.
+ * @gtype: the type of the gutter renderer (must inherit from #GtkSourceGutterRenderer).
+ * @position: the renderers position.
  *
- * Insert @renderer into the gutter. If @renderer is yet unowned then gutter
- * claims its ownership. Otherwise just increases renderer's reference count.
- * @renderer cannot be already inserted to another gutter.
- *
- * Returns: %TRUE if operation succeeded. Otherwise %FALSE.
+ * Inserts @renderer into @gutter at @position.
  *
  * Since: 3.0
  *
- **/
-
-gboolean
-gtk_source_gutter_insert (GtkSourceGutter         *gutter,
-                          GtkSourceGutterRenderer *renderer,
-                          gint                     position)
+ * Returns: (transfer none): a #GtkSourceGutterRenderer
+ */
+GtkSourceGutterRenderer *
+gtk_source_gutter_insert (GtkSourceGutter *gutter,
+                          GType            gtype,
+                          gint             position,
+                          ...)
 {
-	Renderer* internal_renderer;
+	va_list ap;
+	GtkSourceGutterRenderer *ret;
 
-	g_return_val_if_fail (GTK_SOURCE_IS_GUTTER (gutter), FALSE);
-	g_return_val_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER (renderer), FALSE);
-	g_return_val_if_fail (gtk_source_gutter_renderer_get_view (renderer) == NULL, FALSE);
-	g_return_val_if_fail (gtk_source_gutter_renderer_get_window_type (renderer) == GTK_TEXT_WINDOW_PRIVATE, FALSE);
+	g_return_val_if_fail (GTK_IS_SOURCE_GUTTER (gutter), NULL);
+	g_return_val_if_fail (g_type_is_a (gtype, GTK_TYPE_SOURCE_GUTTER_RENDERER), NULL);
 
-	internal_renderer = renderer_new (gutter, renderer, position);
-	append_renderer (gutter, internal_renderer);
+	va_start (ap, position);
+	ret = gtk_source_gutter_insert_valist (gutter, gtype, position, ap);
+	va_end (ap);
 
-	return TRUE;
+	return ret;
+}
+
+/**
+ * gtk_source_gutter_insert_valist:
+ * @gutter: a #GtkSourceGutter.
+ * @gtype: the type of the gutter renderer (must inherit from #GtkSourceGutterRenderer).
+ * @position: the renderer position.
+ * @ap: the parameters.
+ *
+ * Create and insert a new gutter renderer in the gutter. This function is
+ * mostly useful for bindings. Applications should normally use
+ * #gtk_source_gutter_renderer_insert
+ *
+ * Since: 3.0
+ *
+ * Returns: (transfer none): a #GtkSourceGutterRenderer
+ *
+ **/
+GtkSourceGutterRenderer *
+gtk_source_gutter_insert_valist (GtkSourceGutter *gutter,
+                                 GType            gtype,
+                                 gint             position,
+                                 va_list          ap)
+{
+	const gchar *name;
+	GArray *parameters;
+	GObjectClass *klass;
+	GParameter *params;
+	guint num_parameters;
+	GtkSourceGutterRenderer *ret;
+	guint i;
+
+	g_return_val_if_fail (GTK_IS_SOURCE_GUTTER (gutter), NULL);
+	g_return_val_if_fail (g_type_is_a (gtype, GTK_TYPE_SOURCE_GUTTER_RENDERER), NULL);
+
+	parameters = g_array_new (FALSE, TRUE, sizeof (GParameter));
+	klass = g_type_class_ref (gtype);
+
+	num_parameters = 0;
+
+	while ((name = va_arg (ap, const gchar *)) != NULL)
+	{
+		GParameter parameter;
+		GParamSpec *spec;
+		gchar *error = NULL;
+
+		spec = g_object_class_find_property (klass, name);
+
+		if (!spec)
+		{
+			g_warning ("%s: object class `%s' has no property named `%s'",
+			           G_STRFUNC,
+			           g_type_name (gtype),
+			           name);
+
+			break;
+		}
+
+		parameter.name = name;
+
+		G_VALUE_COLLECT_INIT (&parameter.value,
+		                      spec->value_type,
+		                      ap,
+		                      0,
+		                      &error);
+
+		if (error)
+		{
+			g_warning ("%s: %s", G_STRFUNC, error);
+			g_free (error);
+			g_value_unset (&parameter.value);
+
+			break;
+		}
+
+		g_array_append_val (parameters, parameter);
+		++num_parameters;
+	}
+
+	params = (GParameter *)g_array_free (parameters, FALSE);
+
+	ret = gtk_source_gutter_insertv (gutter,
+	                                 gtype,
+	                                 position,
+	                                 num_parameters,
+	                                 params);
+
+	for (i = 0; i < num_parameters; ++i)
+	{
+		g_value_unset (&params[i].value);
+	}
+
+	g_free (params);
+
+	return ret;
+}
+
+/**
+ * gtk_source_gutter_insertv:
+ * @gutter: a #GtkSourceGutter.
+ * @gtype: the type of the gutter renderer (must inherit from #GtkSourceGutterRenderer).
+ * @position: the renderer position.
+ * @num_parameters: the number of parameters.
+ * @parameters: the parameters.
+ *
+ * Create and insert a new gutter renderer in the gutter. This function is
+ * mostly useful for bindings. Applications should normally use
+ * #gtk_source_gutter_renderer_insert
+ *
+ * Since: 3.0
+ *
+ * Returns: (transfer none): a #GtkSourceGutterRenderer
+ *
+ **/
+GtkSourceGutterRenderer *
+gtk_source_gutter_insertv (GtkSourceGutter *gutter,
+                           GType            gtype,
+                           gint             position,
+                           guint            num_parameters,
+                           GParameter      *parameters)
+{
+	GParameter *extended;
+	guint i;
+	gint len;
+	GValue *view;
+	GValue *window_type;
+	Renderer *renderer;
+
+	g_return_val_if_fail (GTK_IS_SOURCE_GUTTER (gutter), NULL);
+	g_return_val_if_fail (g_type_is_a (gtype, GTK_TYPE_SOURCE_GUTTER_RENDERER), NULL);
+
+	len = sizeof (GParameter) * (num_parameters + 2);
+	extended = g_slice_alloc0 (len);
+
+	for (i = 0; i < num_parameters; ++i)
+	{
+		extended[i] = parameters[i];
+	}
+
+	extended[num_parameters].name = "view";
+	view = &(extended[num_parameters].value);
+	g_value_init (view, GTK_TYPE_TEXT_VIEW);
+	g_value_set_object (view, gutter->priv->view);
+
+	extended[num_parameters + 1].name = "window-type";
+	window_type = &(extended[num_parameters + 1].value);
+	g_value_init (window_type, GTK_TYPE_TEXT_WINDOW_TYPE);
+	g_value_set_enum (window_type, gutter->priv->window_type);
+
+	renderer = renderer_new (gutter,
+	                         gtype,
+	                         position,
+	                         num_parameters + 2,
+	                         extended);
+
+	append_renderer (gutter, renderer);
+
+	g_value_unset (view);
+	g_value_unset (window_type);
+
+	g_slice_free1 (len, extended);
+
+	return renderer->renderer;
 }
 
 static gboolean
@@ -738,13 +869,13 @@ gtk_source_gutter_reorder (GtkSourceGutter         *gutter,
 	Renderer *ret;
 	GList *retlist;
 
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER (gutter));
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER (renderer));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER (gutter));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER_RENDERER (renderer));
 
 	if (renderer_find (gutter, renderer, &ret, &retlist))
 	{
 		gutter->priv->renderers =
-			g_list_delete_link (gutter->priv->renderers,
+			g_list_remove_link (gutter->priv->renderers,
 			                    retlist);
 
 		ret->position = position;
@@ -768,15 +899,17 @@ gtk_source_gutter_remove (GtkSourceGutter         *gutter,
 	Renderer *ret;
 	GList *retlist;
 
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER (gutter));
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER (renderer));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER (gutter));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER_RENDERER (renderer));
 
 	if (renderer_find (gutter, renderer, &ret, &retlist))
 	{
 		gutter->priv->renderers =
-			g_list_delete_link (gutter->priv->renderers,
+			g_list_remove_link (gutter->priv->renderers,
 			                    retlist);
 
+		g_object_set (ret->renderer, "view", NULL, NULL);
+
 		update_gutter_size (gutter);
 		renderer_free (ret);
 	}
@@ -794,7 +927,7 @@ gtk_source_gutter_remove (GtkSourceGutter         *gutter,
 void
 gtk_source_gutter_queue_draw (GtkSourceGutter *gutter)
 {
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER (gutter));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER (gutter));
 
 	do_redraw (gutter);
 }
@@ -922,6 +1055,7 @@ on_view_draw (GtkSourceView   *view,
 	GtkTextView *text_view;
 	GArray *sizes;
 	GdkRectangle clip;
+	gint size;
 	gint x, y;
 	gint y1, y2;
 	GArray *numbers;
@@ -941,8 +1075,6 @@ on_view_draw (GtkSourceView   *view,
 	GtkTextIter selection_end;
 	gboolean has_selection;
 	gint idx;
-	GtkStyleContext *style_context;
-	GdkRGBA fg_color;
 
 	window = gtk_source_gutter_get_window (gutter);
 
@@ -989,7 +1121,7 @@ on_view_draw (GtkSourceView   *view,
 	heights = g_array_new (FALSE, FALSE, sizeof (gint));
 	sizes = g_array_new (FALSE, FALSE, sizeof (gint));
 
-	calculate_gutter_size (gutter, sizes);
+	size = calculate_gutter_size (gutter, sizes);
 
 	i = 0;
 	x = 0;
@@ -1020,14 +1152,6 @@ on_view_draw (GtkSourceView   *view,
 	item = gutter->priv->renderers;
 	idx = 0;
 
-	style_context = gtk_widget_get_style_context (GTK_WIDGET (view));
-
-	gtk_style_context_get_color (style_context,
-	                             gtk_widget_get_state (GTK_WIDGET (view)),
-	                             &fg_color);
-
-	gdk_cairo_set_source_rgba (cr, &fg_color);
-
 	while (item)
 	{
 		Renderer *renderer = item->data;
@@ -1235,11 +1359,6 @@ renderer_at_x (GtkSourceGutter *gutter,
 		Renderer *renderer = item->data;
 		gint xpad;
 
-		if (!gtk_source_gutter_renderer_get_visible (renderer->renderer))
-		{
-			continue;
-		}
-
 		w = gtk_source_gutter_renderer_get_size (renderer->renderer);
 
 		gtk_source_gutter_renderer_get_padding (renderer->renderer,
@@ -1277,9 +1396,11 @@ get_renderer_rect (GtkSourceGutter *gutter,
                    GdkRectangle    *rectangle,
                    gint             start)
 {
+	GList *item;
 	gint y;
 	gint ypad;
 
+	item = gutter->priv->renderers;
 	rectangle->x = start;
 
 	gtk_text_view_get_line_yrange (GTK_TEXT_VIEW (gutter->priv->view),
@@ -1572,19 +1693,12 @@ on_view_query_tooltip (GtkSourceView   *view,
 	                                                 tooltip);
 }
 
-static void
-on_view_style_updated (GtkSourceView   *view,
-                       GtkSourceGutter *gutter)
-{
-	gtk_source_gutter_queue_draw (gutter);
-}
-
 void
 gtk_source_gutter_set_padding (GtkSourceGutter *gutter,
                                gint             xpad,
                                gint             ypad)
 {
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER (gutter));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER (gutter));
 
 	if (set_xpad (gutter, xpad, FALSE) || set_ypad (gutter, ypad, FALSE))
 	{
@@ -1597,7 +1711,7 @@ gtk_source_gutter_get_padding (GtkSourceGutter *gutter,
                                gint            *xpad,
                                gint            *ypad)
 {
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER (gutter));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER (gutter));
 
 	if (xpad)
 	{
@@ -1609,32 +1723,3 @@ gtk_source_gutter_get_padding (GtkSourceGutter *gutter,
 		*ypad = gutter->priv->ypad;
 	}
 }
-
-/**
- * gtk_source_gutter_get_renderer_at_pos:
- * @gutter: A #GtkSourceGutter.
- * @x: The x position to get identified.
- * @y: The y position to get identified.
- *
- * Finds the #GtkSourceGutterRenderer at (x, y).
- *
- * Returns: (transfer none): the renderer at (x, y) or %NULL.
- */
-GtkSourceGutterRenderer *
-gtk_source_gutter_get_renderer_at_pos (GtkSourceGutter *gutter,
-                                       gint             x,
-                                       gint             y)
-{
-	Renderer *renderer;
-
-	g_return_val_if_fail (GTK_SOURCE_IS_GUTTER (gutter), NULL);
-
-	renderer = renderer_at_x (gutter, x, NULL, NULL);
-
-	if (renderer == NULL)
-	{
-		return NULL;
-	}
-
-	return renderer->renderer;
-}
