@@ -30,12 +30,12 @@
 
 G_BEGIN_DECLS
 
-#define GTK_SOURCE_TYPE_GUTTER			(gtk_source_gutter_get_type ())
-#define GTK_SOURCE_GUTTER(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_SOURCE_TYPE_GUTTER, GtkSourceGutter))
-#define GTK_SOURCE_GUTTER_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), GTK_SOURCE_TYPE_GUTTER, GtkSourceGutterClass))
-#define GTK_SOURCE_IS_GUTTER(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_SOURCE_TYPE_GUTTER))
-#define GTK_SOURCE_IS_GUTTER_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_SOURCE_TYPE_GUTTER))
-#define GTK_SOURCE_GUTTER_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_SOURCE_TYPE_GUTTER, GtkSourceGutterClass))
+#define GTK_TYPE_SOURCE_GUTTER			(gtk_source_gutter_get_type ())
+#define GTK_SOURCE_GUTTER(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_SOURCE_GUTTER, GtkSourceGutter))
+#define GTK_SOURCE_GUTTER_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_SOURCE_GUTTER, GtkSourceGutterClass))
+#define GTK_IS_SOURCE_GUTTER(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_SOURCE_GUTTER))
+#define GTK_IS_SOURCE_GUTTER_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_SOURCE_GUTTER))
+#define GTK_SOURCE_GUTTER_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_SOURCE_GUTTER, GtkSourceGutterClass))
 
 typedef struct _GtkSourceGutter			GtkSourceGutter;
 typedef struct _GtkSourceGutterClass	GtkSourceGutterClass;
@@ -57,9 +57,24 @@ GType gtk_source_gutter_get_type 		(void) G_GNUC_CONST;
 
 GdkWindow *gtk_source_gutter_get_window 	(GtkSourceGutter         *gutter);
 
-gboolean gtk_source_gutter_insert               (GtkSourceGutter         *gutter,
-                                                 GtkSourceGutterRenderer *renderer,
-                                                 gint                     position);
+GtkSourceGutterRenderer *
+	gtk_source_gutter_insert		(GtkSourceGutter         *gutter,
+                                                 GType                    gtype,
+                                                 gint                     position,
+                                                 ...) G_GNUC_NULL_TERMINATED;
+
+GtkSourceGutterRenderer *
+	gtk_source_gutter_insert_valist         (GtkSourceGutter         *gutter,
+                                                 GType                    gtype,
+                                                 gint                     position,
+                                                 va_list                  ap);
+
+GtkSourceGutterRenderer *
+	gtk_source_gutter_insertv               (GtkSourceGutter         *gutter,
+                                                 GType                    gtype,
+                                                 gint                     position,
+                                                 guint                    num_parameters,
+                                                 GParameter              *parameters);
 
 void gtk_source_gutter_reorder			(GtkSourceGutter	 *gutter,
                                                  GtkSourceGutterRenderer *renderer,
