@@ -1,28 +1,7 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
- * gtksourcegutterrenderertext.c
- * This file is part of GtkSourceView
- *
- * Copyright (C) 2010 - Jesse van den Kieboom
- *
- * GtkSourceView is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * GtkSourceView is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- */
-
 #include "gtksourcegutterrenderertext.h"
 #include "gtksourceview-i18n.h"
 
-#define GTK_SOURCE_GUTTER_RENDERER_TEXT_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_GUTTER_RENDERER_TEXT, GtkSourceGutterRendererTextPrivate))
+#define GTK_SOURCE_GUTTER_RENDERER_TEXT_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_TYPE_SOURCE_GUTTER_RENDERER_TEXT, GtkSourceGutterRendererTextPrivate))
 
 struct _GtkSourceGutterRendererTextPrivate
 {
@@ -35,13 +14,13 @@ struct _GtkSourceGutterRendererTextPrivate
 	guint is_markup : 1;
 };
 
-G_DEFINE_TYPE (GtkSourceGutterRendererText, gtk_source_gutter_renderer_text, GTK_SOURCE_TYPE_GUTTER_RENDERER)
+G_DEFINE_TYPE (GtkSourceGutterRendererText, gtk_source_gutter_renderer_text, GTK_TYPE_SOURCE_GUTTER_RENDERER)
 
 enum
 {
 	PROP_0,
 	PROP_MARKUP,
-	PROP_TEXT
+	PROP_TEXT,
 };
 
 static void
@@ -50,18 +29,16 @@ create_layout (GtkSourceGutterRendererText *renderer,
 {
 	PangoLayout *layout;
 	PangoAttribute *attr;
-	GtkStyleContext *context;
-	GdkRGBA color;
+	GdkColor color;
+	GtkStyle *style;
 	PangoAttrList *attr_list;
 
 	layout = gtk_widget_create_pango_layout (widget, NULL);
 
-	context = gtk_widget_get_style_context (widget);
-	gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &color);
+	style = gtk_widget_get_style (widget);
+	color = style->fg[GTK_STATE_NORMAL];
 
-	attr = pango_attr_foreground_new (color.red * 65535,
-	                                  color.green * 65535,
-	                                  color.blue * 65535);
+	attr = pango_attr_foreground_new (color.red, color.green, color.blue);
 
 	attr->start_index = 0;
 	attr->end_index = G_MAXINT;
@@ -77,8 +54,8 @@ create_layout (GtkSourceGutterRendererText *renderer,
 static void
 gutter_renderer_text_begin (GtkSourceGutterRenderer      *renderer,
                             cairo_t                      *cr,
-                            GdkRectangle                 *background_area,
-                            GdkRectangle                 *cell_area,
+                            const GdkRectangle           *background_area,
+                            const GdkRectangle           *cell_area,
                             GtkTextIter                  *start,
                             GtkTextIter                  *end)
 {
@@ -91,7 +68,7 @@ gutter_renderer_text_begin (GtkSourceGutterRenderer      *renderer,
 
 static void
 center_on (GtkSourceGutterRenderer *renderer,
-           GdkRectangle            *cell_area,
+           const GdkRectangle      *cell_area,
            GtkTextIter             *iter,
            gint                     width,
            gint                     height,
@@ -114,8 +91,8 @@ center_on (GtkSourceGutterRenderer *renderer,
 static void
 gutter_renderer_text_draw (GtkSourceGutterRenderer      *renderer,
                            cairo_t                      *cr,
-                           GdkRectangle                 *background_area,
-                           GdkRectangle                 *cell_area,
+                           const GdkRectangle           *background_area,
+                           const GdkRectangle           *cell_area,
                            GtkTextIter                  *start,
                            GtkTextIter                  *end,
                            GtkSourceGutterRendererState  state)
@@ -130,7 +107,6 @@ gutter_renderer_text_draw (GtkSourceGutterRenderer      *renderer,
 	GtkTextView *view;
 	gint x = 0;
 	gint y = 0;
-	GtkStyleContext *context;
 
 	/* Chain up to draw background */
 	GTK_SOURCE_GUTTER_RENDERER_CLASS (
@@ -212,8 +188,15 @@ gutter_renderer_text_draw (GtkSourceGutterRenderer      *renderer,
 		break;
 	}
 
-	context = gtk_widget_get_style_context (GTK_WIDGET (view));
-	gtk_render_layout (context, cr, x, y, text->priv->cached_layout);
+	gtk_paint_layout (gtk_widget_get_style (GTK_WIDGET (view)),
+	                  cr,
+	                  gtk_widget_get_state (GTK_WIDGET (view)),
+	                  TRUE,
+	                  GTK_WIDGET (view),
+	                  "gtksourcegutterrenderertext",
+	                  x,
+	                  y,
+	                  text->priv->cached_layout);
 }
 
 static void
@@ -284,7 +267,7 @@ gtk_source_gutter_renderer_text_measure (GtkSourceGutterRendererText *renderer,
                                          gint                        *width,
                                          gint                        *height)
 {
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER_TEXT (renderer));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER_RENDERER_TEXT (renderer));
 	g_return_if_fail (text != NULL);
 
 	measure_text (renderer, NULL, text, width, height);
@@ -296,7 +279,7 @@ gtk_source_gutter_renderer_text_measure_markup (GtkSourceGutterRendererText *ren
                                                 gint                        *width,
                                                 gint                        *height)
 {
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER_TEXT (renderer));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER_RENDERER_TEXT (renderer));
 	g_return_if_fail (markup != NULL);
 
 	measure_text (renderer, markup, NULL, width, height);
@@ -425,7 +408,7 @@ gtk_source_gutter_renderer_text_init (GtkSourceGutterRendererText *self)
 GtkSourceGutterRenderer *
 gtk_source_gutter_renderer_text_new ()
 {
-	return g_object_new (GTK_SOURCE_TYPE_GUTTER_RENDERER_TEXT, NULL);
+	return g_object_new (GTK_TYPE_SOURCE_GUTTER_RENDERER_TEXT, NULL);
 }
 
 void
@@ -433,7 +416,7 @@ gtk_source_gutter_renderer_text_set_markup (GtkSourceGutterRendererText *rendere
                                             const gchar                 *markup,
                                             gint                         length)
 {
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER_TEXT (renderer));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER_RENDERER_TEXT (renderer));
 
 	set_text (renderer, markup, length, TRUE);
 }
@@ -443,7 +426,7 @@ gtk_source_gutter_renderer_text_set_text (GtkSourceGutterRendererText *renderer,
                                           const gchar                 *text,
                                           gint                         length)
 {
-	g_return_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER_TEXT (renderer));
+	g_return_if_fail (GTK_IS_SOURCE_GUTTER_RENDERER_TEXT (renderer));
 
 	set_text (renderer, text, length, FALSE);
 }
